#import "WOTWViewController.h"

@implementation WOTWViewController
@synthesize pageSlider;

#pragma mark - Initialization / Memory Management

- (id)init {
    self = [super init];
    if (self) {
		[self loadPDF];
    }
    return self;
}


-(id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self)
    {
        [self loadPDF];
    }
    return self;
}

- (void)dealloc
{
    CGPDFDocumentRelease(bookPDF);
    [pageSlider release];
    [contentsButton release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Resize the book display
    [self->leavesView setFrame:CGRectMake(0.0f, 0.0f, 563.0f, 845.0f)];
    [self->leavesView setCenter:self.view.center];
    
    // Configure the slider
    [self.pageSlider addTarget:self action:@selector(turnPageWithSlider:) forControlEvents:UIControlEventValueChanged];
    self.pageSlider.minimumValue = 0.0;
    self.pageSlider.maximumValue = (float) ([self numberOfPagesInLeavesView:self->leavesView] - 1);
    self.pageSlider.value = self->leavesView.currentPageIndex;

    [self addContentsUIButton];
    [self addTableOfContentsUIView];
    
}

- (void)viewDidUnload {
    [self setPageSlider:nil];
    [contentsButton release], contentsButton = nil;
    [super viewDidUnload];
}

#pragma mark - Custom App Methods

-(void)loadPDF
{
    CFURLRef pdfURL = CFBundleCopyResourceURL(CFBundleGetMainBundle(), CFSTR("WOTW.pdf"), NULL, NULL);
    bookPDF = CGPDFDocumentCreateWithURL((CFURLRef)pdfURL);
    CFRelease(pdfURL);
}

- (void)turnPageWithSlider:(id)sender
{
    int pageIndex = (int) [self.pageSlider value];
    [self.pageSlider setValue:(float)pageIndex];
    self->leavesView.currentPageIndex = pageIndex;
}

- (void)addContentsUIButton
{
    if(!contentsButton)
    {
        // Basic Initialization
        contentsButton = [[UIButton alloc] initWithFrame:CGRectMake(460.0f, 5.0f, 100.0f, 45.0f)];
        [contentsButton setTitle:@"Contents" forState:UIControlStateNormal];
        [contentsButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
        [contentsButton setTitleColor:[UIColor redColor] forState:UIControlStateHighlighted];
        contentsButton.backgroundColor = [UIColor clearColor];
        
        // When the button is tapped, trigger the "displayTableOfContents" method
        [contentsButton addTarget:self action:@selector(displayTableOfContents) forControlEvents:UIControlEventTouchUpInside];
        
        // Hide the button by default as it shouldn't show on the book cover
        contentsButton.hidden = YES;
        
        // Add the button as a subview of LeavesView
        [self->leavesView addSubview:contentsButton];
    }
}

- (void) addTableOfContentsUIView
{
    if(!tableOfContentsView)
    {
        
        // Create the TOC UIView. Will hold all TOC objects as subvies.
        tableOfContentsView = [[UIView alloc] initWithFrame:CGRectMake(0.0f, 0.0, 563.0f, 845.0f)];
        tableOfContentsView.backgroundColor = [UIColor whiteColor];
        
        // Create the Book Title. Center on TOC at top.
        UILabel *bookHeading = [[UILabel alloc] initWithFrame:CGRectMake(170.0f, 30.0f, 300.0f, 40.0f)];
        bookHeading.font = [UIFont fontWithName:@"Arial" size:24.0f];
        bookHeading.text = @"WAR OF THE WORLDS";
        bookHeading.textColor = [UIColor blackColor];
        bookHeading.backgroundColor = [UIColor whiteColor];
        [bookHeading sizeToFit];
        [tableOfContentsView addSubview:bookHeading];
        [bookHeading release];
        
        // Add "BOOK I" subheader as left column header
        UILabel *bookOneSubtitle = [[UILabel alloc] initWithFrame:CGRectMake(105.0f, 70.0f, 150.0f, 40.0f)];
        bookOneSubtitle.font = [UIFont fontWithName:@"Arial" size:22.0f];
        bookOneSubtitle.text = @"BOOK I";
        bookOneSubtitle.textColor = [UIColor blackColor];
        bookOneSubtitle.backgroundColor = [UIColor whiteColor];
        [bookOneSubtitle sizeToFit];
        [tableOfContentsView addSubview:bookOneSubtitle];
        [bookOneSubtitle release];
        
        // Add "BOOK II" subheader as right column header
        UILabel *bookTwoSubtitle = [[UILabel alloc] initWithFrame:CGRectMake(375.0f, 70.0f, 150.0f, 40.0f)];
        bookTwoSubtitle.font = [UIFont fontWithName:@"Arial" size:22.0f];
        bookTwoSubtitle.text = @"BOOK II";
        bookTwoSubtitle.textColor = [UIColor blackColor];
        bookTwoSubtitle.backgroundColor = [UIColor whiteColor];
        [bookTwoSubtitle sizeToFit];
        [tableOfContentsView addSubview:bookTwoSubtitle];
        [bookTwoSubtitle release];
        
        // Create NSDictionary objects from column plist files
        NSString *bookOneFilepath = [[NSBundle mainBundle]
                                     pathForResource:@"BookOne" ofType:@"plist"];
        NSDictionary *bookOneTOC = [[NSDictionary alloc] initWithContentsOfFile:bookOneFilepath];
        
        NSString *bookTwoFilepath = [[NSBundle mainBundle]
                                     pathForResource:@"BookTwo" ofType:@"plist"];
        NSDictionary *bookTwoTOC = [[NSDictionary alloc] initWithContentsOfFile:bookTwoFilepath];
        
        // Iterate over reach NSDictionary object and add UIButtons
        float colXOffset  = 20.0f;
        float colYOffset = 100.0f;
        
        NSArray *sortedKeys1 = [[bookOneTOC allKeys] sortedArrayUsingComparator:^(id obj1, id obj2) {
            
            if ([obj1 integerValue]  < [obj2 integerValue] ) {
                return (NSComparisonResult)NSOrderedAscending;
            }
            
            if ([obj1 integerValue]  > [obj2 integerValue] ) {
                return (NSComparisonResult)NSOrderedDescending;
            }
            return (NSComparisonResult)NSOrderedSame;
        }];
        
        for(NSString *key in sortedKeys1)
        {
            int pageIndex = [key intValue];
            NSString *sectionTitle = [bookOneTOC objectForKey:key];
            
            UIButton *sectionButton = [[UIButton alloc] initWithFrame:CGRectMake(colXOffset, colYOffset, 250.0f, 35.0f)];
            [sectionButton setTitle:sectionTitle forState:UIControlStateNormal];
            sectionButton.tag = pageIndex;
            [sectionButton addTarget:self action:@selector(contentsButtonPressed:) forControlEvents:UIControlEventTouchUpInside];
            sectionButton.enabled = NO;
            
            [sectionButton setBackgroundColor:[UIColor lightGrayColor]];
            
            [sectionButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [tableOfContentsView addSubview:sectionButton];
            [sectionButton release];
            
            colYOffset += 40.0f;
        }
        [bookOneTOC release];
        
        colXOffset  = 293.0f;
        colYOffset = 100.0f;
        
        NSArray *sortedKeys2 = [[bookTwoTOC allKeys] sortedArrayUsingComparator:^(id obj1, id obj2) {
            
            if ([obj1 integerValue]  < [obj2 integerValue] ) {
                return (NSComparisonResult)NSOrderedAscending;
            }
            
            if ([obj1 integerValue]  > [obj2 integerValue] ) {
                return (NSComparisonResult)NSOrderedDescending;
            }
            return (NSComparisonResult)NSOrderedSame;
        }];        
        
        for(NSString *key in sortedKeys2)
        {
            int pageIndex = [key intValue];
            NSString *sectionTitle = [bookTwoTOC objectForKey:key];
            
            UIButton *sectionButton = [[UIButton alloc] initWithFrame:CGRectMake(colXOffset, colYOffset, 250.0f, 35.0f)];
            [sectionButton setTitle:sectionTitle forState:UIControlStateNormal];
            [sectionButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            sectionButton.tag = pageIndex;
            sectionButton.enabled = NO;
            [sectionButton addTarget:self action:@selector(contentsButtonPressed:) forControlEvents:UIControlEventTouchUpInside];
            
            [sectionButton setBackgroundColor:[UIColor lightGrayColor]];
            [tableOfContentsView addSubview:sectionButton];
            [sectionButton release];
            
            colYOffset += 40.0f;
        }
        [bookTwoTOC release];
        
        // Add the TOC directly to the Leaves view, but hide it at "layer" 0
        [self->leavesView insertSubview:tableOfContentsView atIndex:0];
    }
    
}

- (void)displayTableOfContents
{
    // Hide the page slider
    self.pageSlider.hidden = YES;

    // Traverse subviews looking for buttons
    for (UIView *subview in tableOfContentsView.subviews)
    {
        if([subview isKindOfClass:[UIButton class]])
        {
            UIButton *button = (UIButton *) subview;
            button.enabled = YES;
        }
    }
    
    // Animate the transition with a horizontal flip from right to left
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.5f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromRight forView:self->leavesView cache:YES];
    [self->leavesView bringSubviewToFront:tableOfContentsView];
    [UIView commitAnimations];
}

- (void)contentsButtonPressed:(UIButton *)sender
{
    
    // Update the PDF display position
    self->leavesView.currentPageIndex = sender.tag;
    
    // Display the UISlider
    self.pageSlider.hidden = NO;
    self.pageSlider.value = (float) sender.tag; 
    
    // Animate the PDF back to the top of the leavesView subviews
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.5f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:self->leavesView cache:YES];
    [self->leavesView sendSubviewToBack:tableOfContentsView];
    [UIView commitAnimations];
    
    // Disable the UIButtons added to the TOC when not in view
    for (UIView *subview in tableOfContentsView.subviews)
    {
        if([subview isKindOfClass:[UIButton class]])
        {
            UIButton *button = (UIButton *) subview;
            button.enabled = NO;
        }
    }
}

#pragma mark - LeavesViewDataSource methods

- (NSUInteger) numberOfPagesInLeavesView:(LeavesView*)leavesView {
	return CGPDFDocumentGetNumberOfPages(bookPDF);
}

- (void) renderPageAtIndex:(NSUInteger)index inContext:(CGContextRef)ctx {
	CGPDFPageRef page = CGPDFDocumentGetPage(bookPDF, index + 1);
	CGAffineTransform transform = aspectFit(CGPDFPageGetBoxRect(page, kCGPDFMediaBox),
											CGContextGetClipBoundingBox(ctx));
	CGContextConcatCTM(ctx, transform);
	CGContextDrawPDFPage(ctx, page);
}

#pragma mark - LeavesViewDelegate methods

- (void) leavesView:(LeavesView *)leavesView didTurnToPageAtIndex:(NSUInteger)pageIndex
{
    if((int) self.pageSlider.value != pageIndex)
    {
        self.pageSlider.value = (float) pageIndex;
    }
    
    if(pageIndex > 0)
    {
        contentsButton.hidden = NO;
    }
    else
    {
        contentsButton.hidden = YES;
    }
}

@end
